# -*- coding: utf-8 -*-

"""This code is a part of Hydra Toolkit

.. module:: hydratk.extensions.testenv.translation.en.help
   :platform: Unix
   :synopsis: English language translation for TestEnv extension help generator
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""
language = {
    'name': 'English',
    'ISO-639-1': 'en'
}

''' TestEnv Commands '''
help_cmd = {
    'te-install': 'install testing environment database',
    'te-run': 'start testing environment',

    # standalone with option profile testenv
    'install': 'install testing environment database',
    'run': 'start testing environment'
}

''' TestEnv Options '''
help_opt = {
}
